local Skada = Skada

local LB = GetSpellInfo(188196)
local LvB = GetSpellInfo(51505)
local CL = GetSpellInfo(188443)

local Spells = {
	Base = {
		[51505] = true, -- LvB
		[188196] = true, -- LB
		[188443] = true, -- CL
	},
	
	Overloaded = { -- Overloaded SpellID = Base SpellID
		[77451] = 51505, -- LvB Overload
		[45284] = 188196, -- LB Overload
		[45297] = 188443, -- CL Overload
	},
	
	Name = {
		[LB] = 188196, -- LB
		[CL] = 188443, -- CL
		[LvB] = 51505, -- LvB
	}
}

Skada:AddLoadableModule("Damage", nil, function(Skada, L)
	if Skada.db.profile.modulesBlocked.Damage then return end
	
	local mod = Skada:NewModule("Overload damage")
	local playerOverload = Skada:NewModule("Overload spell list")
	local spellsOverload = Skada:NewModule("Overload spell details")
	local overloadTarget = Skada:NewModule("Overload hit targets")

	local sourceDmg
	local function getDPS(set, player, sourceDmg)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceDmg / math.max(1, totaltime)
	end

	local function getRaidOverloadDPS(set)
		if set.time > 0 then
			return set.overloadDamage / math.max(1, set.time)
		else
			local endtime = set.endtime
			if not endtime then
				endtime = time()
			end
			return set.overloadDamage / math.max(1, endtime - set.starttime)
		end
	end

	local function log_damage(set, overload)
		-- Get the player.
		local player = Skada:get_player(set, overload.playerid, overload.playername)
		if player then

			-- Subtract overkill
	--		local amount = math.max(0,overload.amount - overload.overkill)
			-- Or don't. Seems to be the way other meters do it.
			local amount = overload.amount
	--		self:Print(player.name..": "..overload.spellname.." for "..tostring(amount))

			-- Also add to set total and total non overload damage. NOT USED
			-- set.nonOverloadDamage = set.nonOverloadDamage + amount		

			-- Add spell to player if it does not exist.
			-- Add total overloads if not exist
			if overload.overloaded and not player.overloadDamageSpells[Spells.Overloaded[overload.spellid]] then
				player.overloadDamageSpells[Spells.Overloaded[overload.spellid]] = {spellName = overload.spellname, totalHits = 0, damage = 0, totalOverloads = 0, overloadDamage = 0}
			elseif not overload.overloaded and not player.overloadDamageSpells[overload.spellid] then
				player.overloadDamageSpells[overload.spellid] = {spellName = overload.spellname, totalHits = 0, damage = 0, totalOverloads = 0, overloadDamage = 0}
			end
			
			local spell
			
			if overload.overloaded then
				spell = player.overloadDamageSpells[Spells.Overloaded[overload.spellid]]
				
				spell.totalOverloads = spell.totalOverloads + 1
				spell.overloadDamage = spell.overloadDamage + amount
				
				player.totalOverloads = player.totalOverloads + 1
				player.overloadDamage = player.overloadDamage + amount
				
				set.overloadDamage = set.overloadDamage + amount
				
				if spell.max == nil or amount > spell.max then
					spell.max = amount
				end
				
				if (spell.min == nil or amount < spell.min) and not overload.missed then
					spell.min = amount
				end
			else
				spell = player.overloadDamageSpells[overload.spellid]
				
				spell.damage = spell.damage + amount
				player.nonOverloadDamage = player.nonOverloadDamage + amount
				
				spell.totalHits = spell.totalHits + 1
				player.totalHits = player.totalHits + 1
			end
			
			
			-- For now, only save overloadDamage info to current set.
			-- Saving this to Total may become a memory hog deluxe, and besides, it does not make much sense
			-- to see in Total. Why care which particular mob you overloadDamage the most in a whole raid, for example?
			if set == Skada.current and overload.dstname and amount > 0 then
				-- Make sure destination exists in player.
				if not player.overloadDamaged[overload.dstname] then
					player.overloadDamaged[overload.dstname] = 0
				end

				-- Add to destination.
				player.overloadDamaged[overload.dstname] = player.overloadDamaged[overload.dstname] + amount
			end

		end
	end
		
	local overload = {}
		
	local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, smultistrike)
	-- Spell damage.
		if srcGUID ~= dstGUID and (Spells.Base[spellId] or Spells.Overloaded[spellId]) then
			overload.playerid = srcGUID
			overload.playerflags = srcFlags
			overload.dstname = dstName
			overload.playername = srcName
			overload.spellid = spellId
			overload.spellname = spellName
			overload.amount = samount
			
			overload.overloaded = false
			if Spells.Overloaded[spellId] then
				overload.overloaded = true
			end
			
			log_damage(Skada.current, overload)
			log_damage(Skada.total, overload)
		end
	end

	function mod:Update(win, set)
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.nonOverloadDamage > 0 then
				local overloadDps = getDPS(set, player, player.overloadDamage)

				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name

				d.value = player.overloadDamage
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.overloadDamage), self.metadata.columns.Damage,
					Skada:FormatNumber(overloadDps), self.metadata.columns.DPS,
					string.format("%02.1f%%", (player.overloadDamage / set.overloadDamage) * 100), self.metadata.columns.Percent
				)
				if player.overloadDamage > max then
					max = player.overloadDamage
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playerOverload.playerid)
		if player then
			local spell = player.overloadDamageSpells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..spell.spellName)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.overloadDamage / spell.totalOverloads), 255,255,255,255,255,255)
			end
		end
	end
	
	function playerOverload:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerOverload.playerid = id
			playerOverload.title = player.name.."'s Overload Damage"
		end
	end

	function playerOverload:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellID, spell in pairs(player.overloadDamageSpells) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					-- d.label = spell.spellName
					d.id = spellID
					local name, _, icon = GetSpellInfo(spellID)
					d.label = GetSpellInfo(spellID)
					d.icon = icon
					d.spellid = spellID
					d.value = spell.overloadDamage
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(spell.overloadDamage), self.metadata.columns.Damage,
						string.format("%02.1f%%", (spell.overloadDamage / player.overloadDamage) * 100), self.metadata.columns.Percent
					)
					if spell.overloadDamage > max then
						max = spell.overloadDamage
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsOverload.nr + 1
		spellsOverload.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Damage,
			string.format("%02.1f%%", value / spellsOverload.totalHits * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsOverload:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerOverload.playerid)
		spellsOverload.name = label
		spellsOverload.title = player.name.."'s "..label
	end
	--
	function spellsOverload:Update(win, set)
		local player = Skada:find_player(set, playerOverload.playerid)

		if player then
			-- local id = select(7, GetSpellInfo(self.spellID))
			-- local _, _, _, _, _, _, spellID = GetSpellInfo(self.name)
			local spell = player.overloadDamageSpells[Spells.Name[self.name]]
			-- print(self.name," : ",spellID)
		
			if spell then
				spellsOverload.totalHits = spell.totalHits
				spellsOverload.nr = 0
				win.metadata.maxvalue = 0
				
				if spell.totalHits and spell.totalHits > 0 then
					add_detail_bar(win, "Total", spell.totalHits)
				end
				if spell.totalOverloads and spell.totalOverloads > 0 then
					add_detail_bar(win, "Overload", spell.totalOverloads)
				end
			end
		end
	end

	function overloadTarget:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		overloadTarget.playerid = id
		overloadTarget.title = (player and player.name or "Unknown").."'s Overload hit targets"

	end
--
	function overloadTarget:Update(win, set)
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for mob, amount in pairs(player.overloadDamaged) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = mob
					d.id = mob
					d.value = amount
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(amount), self.metadata.columns.Damage,
						string.format("%02.1f%%", (amount / player.overloadDamage) * 100), self.metadata.columns.Percent
					)
					if amount > max then
						max = amount
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerOverload, click2 = overloadTarget, columns = {Damage = true, DPS = true, Percent = true}}
		playerOverload.metadata	= {tooltip = player_tooltip, click1 = spellsOverload, columns = {Damage = true, Percent = true}}
		spellsOverload.metadata = {columns = {Damage = true, Percent = true}}
		overloadTarget.metadata = {showspots = true, columns = {Damage = true, Percent = true}}
			
		Skada:RegisterForCL(SpellDamage, 'SPELL_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_PERIODIC_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_BUILDING_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'RANGE_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.overloadDamageSpells or not player.overloadDamage or not player.totalOverloads or not player.totalHits or not player.nonOverloadDamage then
			player.overloadDamageSpells = player.overloadDamageSpells or {}		-- Stored spell overload info
			player.overloadDamage = player.overloadDamage or 0					-- Total overload damage
			player.totalOverloads = player.totalOverloads or 0
			player.totalHits = player.totalHits or 0
			player.nonOverloadDamage = player.nonOverloadDamage or 0
		end
		
		if not player.overloadDamaged then
			player.overloadDamaged = {}
		end
	end

	function mod:AddToTooltip(set, tooltip)
		GameTooltip:AddDoubleLine("Overload DPS", Skada:FormatNumber(getRaidOverloadDPS(set)), 1,1,1)
	end

	function mod:AddSetAttributes(set)
		if not set.nonOverloadDamage then
			set.nonOverloadDamage = 0
		end
		if not set.overloadDamage then
			set.overloadDamage = 0
		end
	end

	function mod:GetSetSummary(set)
		return Skada:FormatValueText(
			Skada:FormatNumber(set.overloadDamage), self.metadata.columns.Damage,
			Skada:FormatNumber(getRaidOverloadDPS(set)), self.metadata.columns.DPS
		)
	end
end)